/*
 * $Id: PreviewPanel.java,v 1.1.1.1 2002/12/09 17:58:12 sugiyama Exp $
 * Copyright (c) 2002 Hidenori Sugiyama
 */

package to.hslab.jws.capture;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import javax.imageio.*;
import java.io.*;

public class PreviewPanel extends JPanel implements MouseInputListener, ActionListener {
	private BufferedImage capturedImage;
	private Point viewPosition;
	private Point previousCursorPosition;
	private boolean isDragged;
	private JPopupMenu popupMenu;
	private JMenuItem saveMenuItem;
	private JMenuItem closeMenuItem;

	public PreviewPanel() {
		super();

		viewPosition = new Point(0, 0);
		addMouseListener(this);
		addMouseMotionListener(this);
		isDragged = false;

		preparePopupMenu();
	}

	public void setImage(BufferedImage image) {
		capturedImage = image;

		viewPosition.setLocation(0, 0);
	}

	public void paint(Graphics g) {
		int x = viewPosition.x;
		int y = viewPosition.y;

		Dimension panelSize = getSize();
		Dimension imageSize = new Dimension(
			capturedImage.getWidth(),
			capturedImage.getHeight()
		);

		g.drawImage(capturedImage, 0, 0, panelSize.width, panelSize.height,
			viewPosition.x, viewPosition.y,
			viewPosition.x + panelSize.width, viewPosition.y + panelSize.height, this);
	}

	public void update(Graphics g) {
		paint(g);
	}

	public void actionPerformed(ActionEvent event) {
		if (event.getSource() == saveMenuItem) {
			JFileChooser fileChooser = new JFileChooser();
			fileChooser.setFileFilter(new JpegFileFilter());
			int result = fileChooser.showSaveDialog(this);
			if (result == JFileChooser.APPROVE_OPTION) {
				doSave(fileChooser.getSelectedFile());
			}
		} else if (event.getSource() == closeMenuItem) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					getTopLevelAncestor().setVisible(false);
				}
			});
		}
	}

	public void mouseClicked(MouseEvent event) {
	}

	public void mouseEntered(MouseEvent event) {
	}

	public void mouseExited(MouseEvent event) {
	}

	public void mousePressed(MouseEvent event) {
		if (event.getButton() == 1) {
			isDragged = true;
			previousCursorPosition = new Point(event.getX(), event.getY());
		} else {
			popupMenu.show(this, event.getX(), event.getY());
		}
	}

	public void mouseReleased(MouseEvent event) {
		if (event.getButton() == 1) {
			isDragged = false;
			previousCursorPosition = null;
		}
	}

	public void mouseDragged(MouseEvent event) {
		if (!isDragged) {
			return;
		}

		int diffX = event.getX() - previousCursorPosition.x;
		int diffY = event.getY() - previousCursorPosition.y;
		int viewX = viewPosition.x - diffX;
		int viewY = viewPosition.y - diffY;

		Dimension panelSize = getSize();
		Dimension imageSize = new Dimension(
			capturedImage.getWidth(),
			capturedImage.getHeight()
		);
		Dimension maxSize = new Dimension(
			imageSize.width - panelSize.width,
			imageSize.height - panelSize.height
		);

		// Cliping
		if (viewX < 0) {
			viewX = 0;
		} else if (viewX > maxSize.width) {
			viewX = maxSize.width;
		}

		if (viewY < 0) {
			viewY = 0;
		} else if (viewY > maxSize.height) {
			viewY = maxSize.height;
		}

		viewPosition.setLocation(viewX, viewY);
		repaint();

		// update previous position
		previousCursorPosition.setLocation(event.getX(), event.getY());
	}

	public void mouseMoved(MouseEvent event) {
	}

	private void doSave(File file) {
		try {
			ImageIO.write(capturedImage, "jpeg", file);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void preparePopupMenu() {
		saveMenuItem = new JMenuItem("Save");
		saveMenuItem.addActionListener(this);
		closeMenuItem = new JMenuItem("Close");
		closeMenuItem.addActionListener(this);

		popupMenu = new JPopupMenu();
		popupMenu.addSeparator();
		popupMenu.add(saveMenuItem);
		popupMenu.addSeparator();
		popupMenu.add(closeMenuItem);
	}
}
